@@ -108,7 +108,7 @@ function_new_resolve(JSContext *context,
                      JSObject **objp)
 {
     Function *priv;
-    const char *name;
+    char *name;
 
     *objp = NULL;
 
@@ -118,6 +118,7 @@ function_new_resolve(JSContext *context,
     priv = priv_from_js(context, obj);
 
     gjs_debug_jsprop(GJS_DEBUG_GFUNCTION, "Resolve prop '%s' hook obj %p priv %p", name, obj, priv);
+    g_free(name);
 
     if (priv == NULL)
         return JS_TRUE; /* we are the prototype, or have the wrong class */
@@ -398,6 +399,7 @@ gjs_invoke_c_function(JSContext      *context,
     jsval *return_values = NULL;
     guint8 next_rval = 0; /* index into return_values */
     GSList *iter;
+    GIScopeType callback_scope = GI_SCOPE_TYPE_INVALID;
     GjsCallbackTrampoline *callback_trampoline;
     void *destroy_notify;
 
@@ -439,6 +441,8 @@ gjs_invoke_c_function(JSContext      *context,
                                            &callback_trampoline, &destroy_notify)) {
         return JS_FALSE;
     }
+    if (callback_trampoline != NULL)
+        callback_scope = callback_trampoline->scope;
 
     g_callable_info_load_return_type( (GICallableInfo*) function->info, &return_info);
     return_tag = g_type_info_get_tag(&return_info);
@@ -497,8 +501,50 @@ gjs_invoke_c_function(JSContext      *context,
             g_assert_cmpuint(out_args_pos, <, out_args_len);
             g_assert_cmpuint(in_args_pos, <, in_args_len);
 
-            out_arg_cvalues[out_args_pos].v_pointer = NULL;
-            in_arg_cvalues[in_args_pos].v_pointer = &out_arg_cvalues[out_args_pos];
+            if (g_arg_info_is_caller_allocates(&arg_info)) {
+                GITypeTag type_tag;
+                GITypeInfo ainfo;
+
+                g_arg_info_load_type(&arg_info, &ainfo);
+                type_tag = g_type_info_get_tag(&ainfo);
+
+                switch (type_tag) {
+                case GI_TYPE_TAG_INTERFACE:
+                    {
+                        GIBaseInfo* interface_info;
+                        GIInfoType interface_type;
+                        gsize size;
+
+                        interface_info = g_type_info_get_interface(&ainfo);
+                        g_assert(interface_info != NULL);
+
+                        interface_type = g_base_info_get_type(interface_info);
+
+                        if (interface_type == GI_INFO_TYPE_STRUCT) {
+                            size = g_struct_info_get_size((GIStructInfo*)interface_info);
+                        } else if (interface_type == GI_INFO_TYPE_UNION) {
+                            size = g_union_info_get_size((GIUnionInfo*)interface_info);
+                        } else {
+                            failed = TRUE;
+                        }
+
+                        g_base_info_unref((GIBaseInfo*)interface_info);
+
+                        if (!failed) {
+                            in_arg_cvalues[in_args_pos].v_pointer = g_slice_alloc0(size);
+                            out_arg_cvalues[out_args_pos].v_pointer = in_arg_cvalues[in_args_pos].v_pointer;
+                        }
+                        break;
+                    }
+                default:
+                    failed = TRUE;
+                }
+                if (failed)
+                    gjs_throw(context, "Unsupported type %s for (out caller-allocates)", g_type_tag_to_string(type_tag));
+            } else {
+                out_arg_cvalues[out_args_pos].v_pointer = NULL;
+                in_arg_cvalues[in_args_pos].v_pointer = &out_arg_cvalues[out_args_pos];
+            }
             out_args_pos++;
         } else {
             GArgument *in_value;
@@ -620,8 +666,11 @@ gjs_invoke_c_function(JSContext      *context,
     }
 
 release:
-    if (callback_trampoline &&
-        callback_trampoline->scope == GI_SCOPE_TYPE_CALL) {
+    /* Note we saved a copy of the scope in callback_scope above instead
+     * of inspecting callback_trampoline->scope here, because it's possible the
+     * callback was destroyed during the call.
+     */
+    if (callback_trampoline != NULL && callback_scope == GI_SCOPE_TYPE_CALL) {
         gjs_callback_trampoline_free(callback_trampoline);
     }
 
@@ -696,6 +745,37 @@ release:
                 postinvoke_release_failed = TRUE;
             }
 
+            /* For caller-allocates, what happens here is we allocate
+             * a structure above, then gjs_value_from_g_argument calls
+             * g_boxed_copy on it, and takes ownership of that.  So
+             * here we release the memory allocated above.  It would be
+             * better to special case this and directly hand JS the boxed
+             * object and tell gjs_boxed it owns the memory, but for now
+             * this works OK.  We could also alloca() the structure instead
+             * of slice allocating.
+             */
+            if (g_arg_info_is_caller_allocates(&arg_info)) {
+                GITypeTag type_tag;
+                GIBaseInfo* interface_info;
+                GIInfoType interface_type;
+                gsize size;
+
+                type_tag = g_type_info_get_tag(&arg_type_info);
+                g_assert(type_tag == GI_TYPE_TAG_INTERFACE);
+                interface_info = g_type_info_get_interface(&arg_type_info);
+                interface_type = g_base_info_get_type(interface_info);
+                if (interface_type == GI_INFO_TYPE_STRUCT) {
+                    size = g_struct_info_get_size((GIStructInfo*)interface_info);
+                } else if (interface_type == GI_INFO_TYPE_UNION) {
+                    size = g_union_info_get_size((GIUnionInfo*)interface_info);
+                } else {
+                    g_assert_not_reached();
+                }
+
+                g_slice_free1(size, out_arg_cvalues[out_args_pos].v_pointer);
+                g_base_info_unref((GIBaseInfo*)interface_info);
+            }
+
             /* Free GArgument, the jsval should have ref'd or copied it */
             if (!arg_failed)
                 gjs_g_argument_release(context,
@@ -755,32 +835,49 @@ release:
     }
 }
 
-/* this macro was introduced with JSFastNative in 2007 */
-#ifndef JS_ARGV_CALLEE
-#define JS_ARGV_CALLEE(argv)    ((argv)[-2])
-#endif
-
+#ifdef JSFUN_CONSTRUCTOR
 static JSBool
 function_call(JSContext *context,
-              JSObject  *obj, /* "this" object, not the function object */
+              uintN      js_argc,
+              jsval     *vp)
+{
+    jsval *js_argv = JS_ARGV(context, vp);
+    JSObject *object = JS_THIS_OBJECT(context, vp);
+    JSObject *callee = JSVAL_TO_OBJECT(JS_CALLEE(context, vp));
+    jsval retval;
+#else
+static JSBool
+function_call(JSContext *context,
+              JSObject  *object, /* "this" object, not the function object */
               uintN      js_argc,
               jsval     *js_argv,
-              jsval     *rval)
+              jsval     *retval)
 {
+    JSObject *callee = JSVAL_TO_OBJECT(JS_ARGV_CALLEE(js_argv));
+#endif
+    JSBool success;
     Function *priv;
-    JSObject *callee;
-
-    callee = JSVAL_TO_OBJECT(JS_ARGV_CALLEE(js_argv)); /* Callee is the Function object being called */
 
     priv = priv_from_js(context, callee);
     gjs_debug_marshal(GJS_DEBUG_GFUNCTION, "Call callee %p priv %p this obj %p %s", callee, priv,
                       obj, JS_GetTypeName(context,
-                                          JS_TypeOfValue(context, OBJECT_TO_JSVAL(obj))));
+                                          JS_TypeOfValue(context, OBJECT_TO_JSVAL(object))));
 
     if (priv == NULL)
         return JS_TRUE; /* we are the prototype, or have the wrong class */
 
-    return gjs_invoke_c_function(context, priv, obj, js_argc, js_argv, rval);
+
+#ifdef JSFUN_CONSTRUCTOR
+    {
+        jsval retval;
+        success = gjs_invoke_c_function(context, priv, object, js_argc, js_argv, &retval);
+        if (success)
+            JS_SET_RVAL(context, vp, retval);
+    }
+#else
+    success = gjs_invoke_c_function(context, priv, object, js_argc, js_argv, retval);
+#endif
+    return success;
 }
 
 /* If we set JSCLASS_CONSTRUCT_PROTOTYPE flag, then this is called on
@@ -791,24 +888,42 @@ function_call(JSContext *context,
  * identify the prototype as an object of our class with NULL private
  * data.
  */
+#ifdef JSFUN_CONSTRUCTOR
 static JSBool
 function_constructor(JSContext *context,
-                     JSObject  *obj,
+                     uintN      argc,
+                     jsval     *vp)
+{
+    JSObject *object;
+    if (!JS_IsConstructing_PossiblyWithGivenThisObject(context, vp, &object)) {
+        gjs_throw_constructor_error(context);
+        return JS_FALSE;
+    }
+    if (object == NULL)
+        object = JS_NewObjectForConstructor(context, vp);
+#else
+static JSBool
+function_constructor(JSContext *context,
+                     JSObject  *object,
                      uintN      argc,
                      jsval     *argv,
                      jsval     *retval)
 {
+#endif
     Function *priv;
 
     priv = g_slice_new0(Function);
 
     GJS_INC_COUNTER(function);
 
-    g_assert(priv_from_js(context, obj) == NULL);
-    JS_SetPrivate(context, obj, priv);
+    g_assert(priv_from_js(context, object) == NULL);
+    JS_SetPrivate(context, object, priv);
 
     gjs_debug_lifecycle(GJS_DEBUG_GFUNCTION,
-                        "function constructor, obj %p priv %p", obj, priv);
+                        "function constructor, obj %p priv %p", object, priv);
+#ifdef JSFUN_CONSTRUCTOR
+    JS_SET_RVAL(context, vp, OBJECT_TO_JSVAL(object));
+#endif
 
     return JS_TRUE;
 }
