@@ -26,7 +26,7 @@
 #include "ns.h"
 #include "repo.h"
 #include "param.h"
-#include <gjs/gjs.h>
+#include <gjs/gjs-module.h>
 #include <gjs/compat.h>
 
 #include <util/log.h>
@@ -68,10 +68,10 @@ ns_new_resolve(JSContext *context,
                JSObject **objp)
 {
     Ns *priv;
-    const char *name;
+    char *name;
     GIRepository *repo;
     GIBaseInfo *info;
-    JSContext *load_context;
+    JSBool ret = JS_FALSE;
 
     *objp = NULL;
 
@@ -80,17 +80,20 @@ ns_new_resolve(JSContext *context,
 
     /* let Object.prototype resolve these */
     if (strcmp(name, "valueOf") == 0 ||
-        strcmp(name, "toString") == 0)
-        return JS_TRUE;
+        strcmp(name, "toString") == 0) {
+        ret = JS_TRUE;
+        goto out;
+    }
 
     priv = priv_from_js(context, obj);
     gjs_debug_jsprop(GJS_DEBUG_GNAMESPACE, "Resolve prop '%s' hook obj %p priv %p", name, obj, priv);
 
-    if (priv == NULL)
-        return JS_TRUE; /* we are the prototype, or have the wrong class */
+    if (priv == NULL) {
+        ret = JS_TRUE; /* we are the prototype, or have the wrong class */
+        goto out;
+    }
 
-    load_context = gjs_runtime_get_load_context(JS_GetRuntime(context));
-    JS_BeginRequest(load_context);
+    JS_BeginRequest(context);
 
     repo = g_irepository_get_default();
 
@@ -99,23 +102,23 @@ ns_new_resolve(JSContext *context,
         /* Special-case fallback hack for GParamSpec */
         if (strcmp(name, "ParamSpec") == 0 &&
             strcmp(priv->namespace, "GLib") == 0) {
-            gjs_define_param_class(load_context,
-                                   obj,
-                                   NULL);
-            if (gjs_move_exception(load_context, context)) {
-                JS_EndRequest(load_context);
-                return JS_FALSE;
+            if (!gjs_define_param_class(context,
+                                        obj,
+                                        NULL)) {
+                JS_EndRequest(context);
+                goto out;
             } else {
                 *objp = obj; /* we defined the property in this object */
-                JS_EndRequest(load_context);
-                return JS_TRUE;
+                JS_EndRequest(context);
+                ret = JS_TRUE;
+                goto out;
             }
         } else {
             gjs_throw(context,
                       "No symbol '%s' in namespace '%s'",
                       name, priv->namespace);
-            JS_EndRequest(load_context);
-            return JS_FALSE;
+            JS_EndRequest(context);
+            goto out;
         }
     }
 
@@ -125,27 +128,22 @@ ns_new_resolve(JSContext *context,
               g_base_info_get_name(info),
               g_base_info_get_namespace(info));
 
-    if (gjs_define_info(load_context, obj, info)) {
+    if (gjs_define_info(context, obj, info)) {
         g_base_info_unref(info);
         *objp = obj; /* we defined the property in this object */
-        JS_EndRequest(load_context);
-        return JS_TRUE;
+        ret = JS_TRUE;
     } else {
         gjs_debug(GJS_DEBUG_GNAMESPACE,
                   "Failed to define info '%s'",
                   g_base_info_get_name(info));
 
         g_base_info_unref(info);
-
-        if (!gjs_move_exception(load_context, context)) {
-            /* set an exception if none was set */
-            gjs_throw(context,
-                         "Defining info failed but no exception set");
-        }
-
-        JS_EndRequest(load_context);
-        return JS_FALSE;
     }
+    JS_EndRequest(context);
+
+ out:
+    g_free(name);
+    return ret;
 }
 
 /* If we set JSCLASS_CONSTRUCT_PROTOTYPE flag, then this is called on
@@ -156,23 +154,23 @@ ns_new_resolve(JSContext *context,
  * identify the prototype as an object of our class with NULL private
  * data.
  */
-static JSBool
-ns_constructor(JSContext *context,
-               JSObject  *obj,
-               uintN      argc,
-               jsval     *argv,
-               jsval     *retval)
+GJS_NATIVE_CONSTRUCTOR_DECLARE(ns)
 {
+    GJS_NATIVE_CONSTRUCTOR_VARIABLES(ns)
     Ns *priv;
 
+    GJS_NATIVE_CONSTRUCTOR_PRELUDE(ns);
+
     priv = g_slice_new0(Ns);
 
     GJS_INC_COUNTER(ns);
 
-    g_assert(priv_from_js(context, obj) == NULL);
-    JS_SetPrivate(context, obj, priv);
+    g_assert(priv_from_js(context, object) == NULL);
+    JS_SetPrivate(context, object, priv);
 
-    gjs_debug_lifecycle(GJS_DEBUG_GNAMESPACE, "ns constructor, obj %p priv %p", obj, priv);
+    gjs_debug_lifecycle(GJS_DEBUG_GNAMESPACE, "ns constructor, obj %p priv %p", object, priv);
+
+    GJS_NATIVE_CONSTRUCTOR_FINISH(ns);
 
     return JS_TRUE;
 }
@@ -241,7 +239,7 @@ ns_new(JSContext    *context,
     Ns *priv;
 
     /* put constructor in the global namespace */
-    global = JS_GetGlobalObject(context);
+    global = gjs_get_import_global(context);
 
     if (!gjs_object_has_property(context, global, gjs_ns_class.name)) {
         JSObject *prototype;
@@ -256,7 +254,7 @@ ns_new(JSContext    *context,
                                   * none - just name the prototype like
                                   * Math - rarely correct)
                                   */
-                                 ns_constructor,
+                                 gjs_ns_constructor,
                                  /* number of constructor args */
                                  0,
                                  /* props of prototype */
@@ -276,7 +274,7 @@ ns_new(JSContext    *context,
                   gjs_ns_class.name, prototype);
     }
 
-    ns = JS_ConstructObject(context, &gjs_ns_class, NULL, NULL);
+    ns = JS_ConstructObject(context, &gjs_ns_class, NULL, global);
     if (ns == NULL)
         gjs_fatal("No memory to create ns object");
 
