@@ -238,25 +238,6 @@ gjs_string_from_filename(JSContext  *context,
 
 /**
  * gjs_string_get_ascii:
- * @value: a jsval
- *
- * Get the char array in the JSString contained in @value.
- * The string is expected to be encoded in ASCII, otherwise
- * you will get garbage out. See the documentation for
- * JS_GetStringBytes() for more details.
- *
- * Returns: an ASCII C string
- **/
-const char*
-gjs_string_get_ascii(jsval value)
-{
-    g_return_val_if_fail(JSVAL_IS_STRING(value), NULL);
-
-    return JS_GetStringBytes(JSVAL_TO_STRING(value));
-}
-
-/**
- * gjs_string_get_ascii_checked:
  * @context: a JSContext
  * @value: a jsval
  *
@@ -266,16 +247,32 @@ gjs_string_get_ascii(jsval value)
  *
  * Returns: an ASCII C string or %NULL on error
  **/
-const char*
-gjs_string_get_ascii_checked(JSContext       *context,
-                             jsval            value)
+char*
+gjs_string_get_ascii(JSContext       *context,
+                     jsval            value)
 {
+    JSString *str;
+
     if (!JSVAL_IS_STRING(value)) {
         gjs_throw(context, "A string was expected, but value was not a string");
         return NULL;
     }
 
-    return JS_GetStringBytes(JSVAL_TO_STRING(value));
+    str = JSVAL_TO_STRING(value);
+
+#ifdef HAVE_JS_GETSTRINGBYTES
+    return g_strdup(JS_GetStringBytes(str));
+#else
+    char *ascii;
+    size_t len = JS_GetStringEncodingLength(context, str);
+    if (len == (size_t)(-1))
+        return NULL;
+
+    ascii = g_malloc((len + 1) * sizeof(char));
+    JS_EncodeStringToBuffer(str, ascii, len);
+    ascii[len] = '\0';
+    return ascii;
+#endif
 }
 
 static JSBool
@@ -310,13 +307,14 @@ gjs_string_get_binary_data(JSContext       *context,
                            char           **data_p,
                            gsize           *len_p)
 {
-    char *js_data;
+    JSString *str;
 
     JS_BeginRequest(context);
 
     if (!JSVAL_IS_STRING(value)) {
         gjs_throw(context,
                   "Value is not a string, can't return binary data from it");
+        JS_EndRequest(context);
         return JS_FALSE;
     }
 
@@ -325,12 +323,25 @@ gjs_string_get_binary_data(JSContext       *context,
         return JS_FALSE;
     }
 
-    js_data = JS_GetStringBytes(JSVAL_TO_STRING(value));
+    str = JSVAL_TO_STRING(value);
+
+#ifdef HAVE_JS_GETSTRINGBYTES
+    const char *js_data = JS_GetStringBytes(str);
+
     /* GetStringLength returns number of 16-bit jschar;
      * we stored binary data as 1 byte per jschar
      */
-    *len_p = JS_GetStringLength(JSVAL_TO_STRING(value));
+    *len_p = JS_GetStringLength(str);
     *data_p = g_memdup(js_data, *len_p);
+#else
+    *len_p = JS_GetStringEncodingLength(context, str);
+    if (*len_p == (gsize)(-1))
+        return JS_FALSE;
+
+    *data_p = g_malloc((*len_p + 1) * sizeof(char));
+    JS_EncodeStringToBuffer(str, *data_p, *len_p);
+    (*data_p)[*len_p] = '\0';
+#endif
 
     JS_EndRequest(context);
 
@@ -432,15 +443,27 @@ gjs_string_get_uint16_data(JSContext       *context,
 JSBool
 gjs_get_string_id (JSContext       *context,
                    jsid             id,
-                   const char     **name_p)
+                   char           **name_p)
 {
     jsval id_val;
+    JSString *str;
 
     if (!JS_IdToValue(context, id, &id_val))
         return JS_FALSE;
 
     if (JSVAL_IS_STRING(id_val)) {
-        *name_p = JS_GetStringBytes(JSVAL_TO_STRING(id_val));
+        str = JSVAL_TO_STRING(id_val);
+#ifdef HAVE_JS_GETSTRINGBYTES
+        *name_p = g_strdup(JS_GetStringBytes(str));
+#else
+        size_t len = JS_GetStringEncodingLength(context, str);
+        if (len == (size_t)(-1))
+            return JS_FALSE;
+
+        *name_p = g_malloc((len + 1) * sizeof(char));
+        JS_EncodeStringToBuffer(str, *name_p, len);
+        (*name_p)[len] = '\0';
+#endif
         return JS_TRUE;
     } else {
         *name_p = NULL;
@@ -448,6 +471,31 @@ gjs_get_string_id (JSContext       *context,
     }
 }
 
+/**
+ * gjs_unichar_from_string:
+ * @string: A string
+ * @result: (out): A unicode character
+ *
+ * If successful, @result is assigned the Unicode codepoint
+ * corresponding to the first full character in @string.  This
+ * function handles characters outside the BMP.
+ *
+ * If @string is empty, @result will be 0.  An exception will
+ * be thrown if @string can not be represented as UTF-8.
+ */
+gboolean
+gjs_unichar_from_string (JSContext *context,
+                         jsval      value,
+                         gunichar  *result)
+{
+    char *utf8_str;
+    if (gjs_string_to_utf8(context, value, &utf8_str)) {
+        *result = g_utf8_get_char(utf8_str);
+        g_free(utf8_str);
+        return TRUE;
+    }
+    return FALSE;
+}
 
 #if GJS_BUILD_TESTS
 #include "unit-test-utils.h"
@@ -486,14 +534,19 @@ gjstest_test_func_gjs_jsapi_util_string_get_ascii(void)
     const char *ascii_string = "Hello, world";
     JSString  *js_string;
     jsval      void_value;
+    char *test;
 
     _gjs_unit_test_fixture_begin(&fixture);
     context = fixture.context;
 
     js_string = JS_NewStringCopyZ(context, ascii_string);
-    g_assert(g_str_equal(gjs_string_get_ascii(STRING_TO_JSVAL(js_string)), ascii_string));
+    test = gjs_string_get_ascii(context, STRING_TO_JSVAL(js_string));
+    g_assert(g_str_equal(test, ascii_string));
+    g_free(test);
     void_value = JSVAL_VOID;
-    g_assert(gjs_string_get_ascii_checked(context, void_value) == NULL);
+    test = gjs_string_get_ascii(context, void_value);
+    g_assert(test == NULL);
+    g_free(test);
     g_assert(JS_IsExceptionPending(context));
 
     _gjs_unit_test_fixture_finish(&fixture);
