@@ -267,7 +267,7 @@ void        gjs_log_object_props             (JSContext       *context,
                                               GjsDebugTopic    topic,
                                               const char      *prefix);
 #endif
-const char* gjs_value_debug_string           (JSContext       *context,
+char*       gjs_value_debug_string           (JSContext       *context,
                                               jsval            value);
 void        gjs_explain_scope                (JSContext       *context,
                                               const char      *title);
@@ -311,8 +311,7 @@ JSBool      gjs_string_from_filename         (JSContext       *context,
                                               const char      *filename_string,
                                               gssize           n_bytes,
                                               jsval           *value_p);
-const char* gjs_string_get_ascii             (jsval            value);
-const char* gjs_string_get_ascii_checked     (JSContext       *context,
+char*       gjs_string_get_ascii             (JSContext       *context,
                                               jsval            value);
 JSBool      gjs_string_get_binary_data       (JSContext       *context,
                                               jsval            value,
@@ -328,7 +327,13 @@ JSBool      gjs_string_get_uint16_data       (JSContext       *context,
                                               gsize           *len_p);
 JSBool      gjs_get_string_id                (JSContext       *context,
                                               jsid             id,
-                                              const char     **name_p);
+                                              char           **name_p);
+
+
+gboolean    gjs_unichar_from_string          (JSContext       *context,
+                                              jsval            string,
+                                              gunichar        *result);
+
 const char* gjs_get_type_name                (jsval            value);
 
 jsval       gjs_date_from_time_t             (JSContext *context, time_t time);
